
// EMS Class headers
// Copyright 1993, 1994 Scott Diorio
// All rights reserved
//
// ----------------------------------------------------------------------
//
// Revision History
//
//
// Beta Version  10/20/93
//
// Version 1.0   12/30/93
//  -initial version
//
// Version 1.1   1/6/94
//  -fixed bugs working with small buffers
//  -enhanced error reporting facilities
//  -speeded up single character access within EMSbuf
//
// Version 1.2  1/12/94
//  -Fixed bugs in all 16K boundary conditions. Added comprehensive
//   test case to test all conditions.
//  -Added operator =, and copy constructor for EMSbuf
//  -Removed several overloaded operators making type checking more
//   normalized to standard C++ usage
//  -Added a memcmp function
//  -Removed several redundant mem functions, all function that use an
//   EMSbuf are now called with a reference (&)
//  -Added Resize(int) function
//  -Changed function name PagesFree() to AppPagesFree() to avoid
//   confusion with SystemPagesFree()
//
// Version 1.3  1/17/94
//  -Added functions DumpMap(), DumpHandles(), and DumpAll() for display
//   of EMS usage in the debugging cycle
// ----------------------------------------------------------------------

// This code is copyrighted and may not be modified without the prior
// written consent of the author.
//
//
//
// EMS Kernel Object and EMS buffer object
//
//

#ifndef _EMSKERNEL.HPP_
#define _EMSKERNEL.HPP_

#include <iostream.h>

#define MAX_EMS_HANDLES 32768
#define EMSPAGESIZE 0x4000

// EMM memory system ERRORS -see "Advanced MSDOS programming" by Ray Duncan
#define EMS_SOFTWARE        0x80
#define EMS_HARDWARE        0x81
#define EMS_BUSY            0x82
#define EMS_HANDLE_INVALID  0x83
#define EMS_FUNCTION_UNDEF  0x84
#define EMS_NO_HANDLES      0x85
#define EMS_SAVERESTORE_MAP 0x86
#define EMS_NOT_ENOUGH_SYSHANDLES 0x87
#define EMS_NOT_ENOUGH_APPHANDLES 0x88
#define EMS_ZERO_PAGES      0x89
#define EMS_PAGE_OUTSIDE_RANGE 0x8A
#define EMS_ILLEGAL_PAGE_NUM 0x8B
#define EMS_PAGE_MAPPING_STATE_FULL 0x8C
#define EMS_NO_SAVE_AREA 0x8D
#define EMS_SAVE_ALREADY_CALLED 0x8E
#define EMS_PARAM_NOT_DEFINED 0x8F

// Errors for EmsKernel & EMSbuf
#define EMS_NOT_INSTALLED    1
#define EMS_NOT_ALLOCATED    2
#define EMSKERNEL_NO_CONVMEM 3
#define EMSBUF_INDEX_RANGE   4
#define EMSBUF_READ          5
#define EMSBUF_WRITE   6
#define EMSBUF_MEMSET  7
#define EMSBUF_MEMCPY  8
#define EMSBUF_MEMCMP  9

#define EMSCLASS_ERRCNT 9

struct EMS_handle
{
	int id;
	int pagecnt;
	unsigned long size;
	int *pages;
};

struct EMSoffset
{
	unsigned page;
	unsigned offset;
};

class EMSbuf;
class EmsKernel
{
	protected:
		static int EmsClassHandle;
		static char maplist[256];
		static char active[4];
		static int Allocated;
		static int searchfree;
		static unsigned *systempageinfo;
		static EMS_handle **usagetable;
		static unsigned kernelfreepages;
		static unsigned kernelpages;
		static int _EmsKernelInstansiation_;
		static int classcnt;
		static int debugstate;
		static int nextactive;
		static int errlevel;
		static int Err;

		inline int SingleMap(int,int);

		int Free(void);
		int FirstFree(void);
		int NextFree(void);
		int Installed(void);
		int SetEnvironmentEMShandle(int);
		int Map(int,int,int,int);
		int FreeHandleOnDisk(void);
		EMS_handle GetSpace(unsigned long);
		void FreeSpace(EMS_handle &);
		int Version(void);
		void SetErrlevel(int);
		void ErrMsg(int);
		int Save(void);
		int Restore(void);
	public:
		EmsKernel(int debug=0,int err=0);
		~EmsKernel(void);
		int SystemPagesFree(void);
		int AppPagesFree(void);
		int Alloc(int);
		void DumpMap(void);
		void DumpHandles(void);
		void DumpAll(void);
};

class EMSbuf : public EmsKernel
{
	protected:
		EMS_handle handle;
		unsigned long offset;
		int lastmap;

	public:
		EMSbuf(void);
		EMSbuf(EMSbuf &);

		EMSbuf(unsigned long);
		~EMSbuf(void);
		void Resize(unsigned long);
		unsigned long Size(void);

		// overloaded operators ------------------------------------------

		inline char& operator[](unsigned long index);  // array indexing

		inline EMSbuf& operator+(unsigned long);

		EMSbuf& operator=(EMSbuf &);
		EMSbuf& operator& (EMSbuf &);
		operator char *();

		//----------------------------------------------------------------

		friend ostream& operator << ( ostream&, EMSbuf &);
		friend unsigned long read(int, EMSbuf &, unsigned long);
		friend unsigned long write(int, EMSbuf &, unsigned long);

		friend EMSbuf& memset(EMSbuf &, int, unsigned long);

		friend EMSbuf& memcpy(EMSbuf &, void *, size_t);
		friend void *  memcpy(void *, EMSbuf &, size_t);
		friend void    memcpy(EMSbuf &, EMSbuf &, unsigned long);

		friend int memcmp(EMSbuf &, void *, size_t);
		friend int memcmp(void *, EMSbuf &, size_t);
		friend int memcmp(EMSbuf &, EMSbuf &, unsigned long);
};


// function prototypes

inline EMSoffset GetPageOffsetLocation(unsigned long);
unsigned long read(int, EMSbuf &, unsigned long);
unsigned long write(int, EMSbuf &, unsigned long);
EMSbuf& memset(EMSbuf &, int, unsigned long);
EMSbuf& memcpy(const EMSbuf &, void *, size_t);
void *  memcpy(void *, EMSbuf &, size_t);
void    memcpy(EMSbuf &, EMSbuf &, unsigned long);
int memcmp(const EMSbuf &, void *, size_t);
int	memcmp(void *, EMSbuf &, size_t);
int memcmp(EMSbuf &, EMSbuf &, unsigned long);

extern char* PageFrame;
extern char termchar;
#endif
